/*
 * vinagre-rdp-plugin.c
 * This file is part of vinagre
 *
 * Copyright (C) 2010 Jonh Wendell <wendell@bani.com.br>
 * 
 * vinagre-rdp-plugin.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * vinagre-rdp-plugin.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>
#include <gmodule.h>

#include <vinagre/vinagre-debug.h>

#include "vinagre-rdp-plugin.h"
#include "vinagre-rdp-connection.h"
#include "vinagre-rdp-tab.h"

#define DEFAULT_WIDTH   800
#define DEFAULT_HEIGHT  600
#define MIN_SIZE          1
#define MAX_SIZE       8192

#define VINAGRE_RDP_PLUGIN_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), VINAGRE_TYPE_RDP_PLUGIN, VinagreRdpPluginPrivate))
  
VINAGRE_PLUGIN_REGISTER_TYPE(VinagreRdpPlugin, vinagre_rdp_plugin)

static void
impl_activate (VinagrePlugin *plugin,
               VinagreWindow *window)
{
  vinagre_debug_message (DEBUG_PLUGINS, "VinagreRdpPlugin Activate");
}

static void
impl_deactivate  (VinagrePlugin *plugin,
                  VinagreWindow *window)
{
  vinagre_debug_message (DEBUG_PLUGINS, "VinagreRdpPlugin Deactivate");
}

static void
impl_update_ui (VinagrePlugin *plugin,
                VinagreWindow *window)
{
  vinagre_debug_message (DEBUG_PLUGINS, "VinagreRdpPlugin Update UI");
}

static const GOptionEntry vinagre_rdp_args[] =
{
  { NULL }
};

static GSList *
impl_get_context_groups (VinagrePlugin *plugin)
{
  GOptionGroup *group;
  GSList       *groups = NULL;

  vinagre_debug_message (DEBUG_PLUGINS, "VinagreRdpPlugin Get Context Group");

  group = g_option_group_new ("rdp",
			      /* Translators: this is a command line option (run vinagre --help) */
			      _("RDP Options:"),
			      /* Translators: this is a command line option (run vinagre --help) */
			      _("Show RDP Options"),
			      NULL,
			      NULL);
  g_option_group_add_entries (group, vinagre_rdp_args);

  groups = g_slist_append (groups, group);

  return groups;
}


static const gchar *
impl_get_protocol (VinagrePlugin *plugin)
{
  return "rdp";
}

static gchar **
impl_get_public_description (VinagrePlugin *plugin)
{
  gchar **result = g_new (gchar *, 3);

  result[0] = g_strdup (_("RDP"));
  /* Translators: This is a description of the RDP protocol. It appears in the Connect dialog. */
  result[1] = g_strdup (_("Access MS Windows remote desktops"));
  result[2] = NULL;

  return result;
}

static const gchar *
impl_get_mdns_service (VinagrePlugin *plugin)
{
  return "_rdp._tcp";
}

static VinagreConnection *
impl_new_connection (VinagrePlugin *plugin)
{
  return vinagre_rdp_connection_new ();
}

static GtkWidget *
impl_new_tab (VinagrePlugin   *plugin,
	      VinagreConnection *conn,
	      VinagreWindow     *window)
{
  return vinagre_rdp_tab_new (conn, window);
}

static gint
impl_get_default_port (VinagrePlugin *plugin)
{
  return 3389;
}

static void
vinagre_rdp_plugin_init (VinagreRdpPlugin *plugin)
{
  vinagre_debug_message (DEBUG_PLUGINS, "VinagreRdpPlugin initializing");
}

static GtkWidget *
impl_get_connect_widget (VinagrePlugin *plugin, VinagreConnection *conn)
{
  GtkWidget *label, *u_entry, *d_entry, *spin_button, *check;
  GtkTable  *table;
  gchar     *str;

  table = GTK_TABLE (gtk_table_new (2, 2, FALSE));
  gtk_table_set_row_spacings (table, 6);

  str = g_strdup_printf ("<b>%s</b>", _("RDP Options"));
  label = gtk_label_new (str);
  g_free (str);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_table_attach_defaults (table, label, 0, 2, 0, 2);


  label = gtk_label_new ("  ");
  gtk_table_attach (table, label, 2, 3, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);

  label = gtk_label_new ("  ");
  gtk_table_attach (table, label, 2, 3, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);

  label = gtk_label_new ("  ");
  gtk_table_attach (table, label, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);


  /* Scaling check button */
  check = gtk_check_button_new_with_mnemonic (_("_Scaling"));
  g_object_set_data (G_OBJECT (table), "scaling", check);
  gtk_table_attach_defaults (table, check, 1, 3, 2, 3);
  if (VINAGRE_IS_CONNECTION (conn))
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
                                  vinagre_rdp_connection_get_scaling (VINAGRE_RDP_CONNECTION (conn)));



  label = gtk_label_new_with_mnemonic (_("_Username:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (table, label, 1, 2, 3, 4);

  u_entry = gtk_entry_new ();
  /* Translators: This is the tooltip for the username field in a RDP connection */
  gtk_widget_set_tooltip_text (u_entry, _("Optional. If blank, your username will be used. Also, it can be supplied in the Host field above, in the form username@hostname."));
  g_object_set_data (G_OBJECT (table), "username_entry", u_entry);
  gtk_table_attach_defaults (table, u_entry, 2, 3, 3, 4);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label), u_entry);
  if (conn != NULL)
    gtk_entry_set_text (GTK_ENTRY (u_entry), vinagre_connection_get_username (conn));
  gtk_entry_set_activates_default (GTK_ENTRY (u_entry), TRUE);


  label = gtk_label_new_with_mnemonic (_("_Domain:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (table, label, 1, 2, 4, 5);

  d_entry = gtk_entry_new ();
  /* Translators: This is the tooltip for the domain field in a RDP connection */
  gtk_widget_set_tooltip_text (d_entry, _("Optional"));
  g_object_set_data (G_OBJECT (table), "domain_entry", d_entry);
  gtk_table_attach_defaults (table, d_entry, 2, 3, 4, 5);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label), d_entry);
  if (conn != NULL)
    gtk_entry_set_text (GTK_ENTRY (d_entry), vinagre_connection_get_domain (conn));
  gtk_entry_set_activates_default (GTK_ENTRY (d_entry), TRUE);


  /* Host width */
  label = gtk_label_new_with_mnemonic (_("_Width:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (table, label, 1, 2, 5, 6);

  spin_button = gtk_spin_button_new_with_range (MIN_SIZE, MAX_SIZE, 1);
  /* Translators: This is the tooltip for the width field in a RDP connection */
  gtk_widget_set_tooltip_text (spin_button, _("Set width of the remote desktop"));
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_button), DEFAULT_WIDTH);
  g_object_set_data (G_OBJECT (table), "width_spin_button", spin_button);
  gtk_table_attach_defaults (table, spin_button, 2, 3, 5, 6);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_button);
  gtk_entry_set_activates_default (GTK_ENTRY (spin_button), TRUE);


  /* Host height */
  label = gtk_label_new_with_mnemonic (_("_Height:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_table_attach_defaults (table, label, 1, 2, 6, 7);

  spin_button = gtk_spin_button_new_with_range (MIN_SIZE, MAX_SIZE, 1);
  /* Translators: This is the tooltip for the height field in a RDP connection */
  gtk_widget_set_tooltip_text (spin_button, _("Set height of the remote desktop"));
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin_button), DEFAULT_HEIGHT);
  g_object_set_data (G_OBJECT (table), "height_spin_button", spin_button);
  gtk_table_attach_defaults (table, spin_button, 2, 3, 6, 7);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_button);
  gtk_entry_set_activates_default (GTK_ENTRY (spin_button), TRUE);


  return GTK_WIDGET (table);
}

static void
vinagre_rdp_plugin_finalize (GObject *object)
{
  vinagre_debug_message (DEBUG_PLUGINS, "VinagreRdpPlugin finalizing");

  G_OBJECT_CLASS (vinagre_rdp_plugin_parent_class)->finalize (object);
}

static void
vinagre_rdp_plugin_class_init (VinagreRdpPluginClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  VinagrePluginClass *plugin_class = VINAGRE_PLUGIN_CLASS (klass);

  object_class->finalize   = vinagre_rdp_plugin_finalize;

  plugin_class->activate   = impl_activate;
  plugin_class->deactivate = impl_deactivate;
  plugin_class->update_ui  = impl_update_ui;
  plugin_class->get_protocol  = impl_get_protocol;
  plugin_class->get_context_groups = impl_get_context_groups;
  plugin_class->get_public_description  = impl_get_public_description;
  plugin_class->new_connection = impl_new_connection;

  plugin_class->get_mdns_service  = impl_get_mdns_service;
  plugin_class->new_tab = impl_new_tab;
  plugin_class->get_connect_widget = impl_get_connect_widget;
  plugin_class->get_default_port = impl_get_default_port;
}

/* vim: set ts=8: */
